import os

from xml.etree import ElementTree
from xml.etree.ElementTree import Comment, Element, SubElement, tostring
from xml.dom import minidom

import feedparser


class FeedList:
    def __init__(self):
        self.fname = os.path.join(os.path.expanduser('~'),
                                  'Subscriptions.opml')
        if not os.path.exists(self.fname):
            self.initialize()

        with open(self.fname, 'rt') as f:
            self.tree = ElementTree.parse(f)
            self.root = self.tree.getroot()
            self.body = self.root.find('body')

    def reset(self):
        os.unlink(self.fname)
        self.initialize()
        print('Initialized!')

    def initialize(self):
        root = Element('opml', version='2.0')
        head = Element('head')
        title = SubElement(head, 'title', text='Subscriptions.opml')
        body = Element('body')
        root.extend([head, body])

        self._write_tree(root)

    def _write_tree(self, elem):
        string = tostring(elem, 'utf-8')
        parsed = minidom.parseString(string)

        with open(self.fname, mode='wt') as f:
            parsed.writexml(f)

    def get_categories(self):
        cats = []

        for node in self.tree.findall('.//outline'):
            url = node.attrib.get('xmlUrl')

            if url:
                pass
            else:
                cat = node.attrib.get('text')
                cats.append(cat)

        return cats

    def get_subscriptions(self):
        urls = []

        for node in self.tree.findall('.//outline/outline'):
            url = node.attrib.get('xmlUrl')
            urls.append(url)

        return urls

    def _parse(self, url):
        d = feedparser.parse(url)
        return d

    def add(self, url, category='uncategorized'):
        subs = self.get_subscriptions()

        if url in subs:
            # check if url already subscribed
            if url in subs:
                print(f'"{url}" has already subscribed.')
        else:
            site = self._parse(url)

            if not category in self.get_categories():
                # create a new category if no exists
                # then add a new subscription
                cat = SubElement(self.body,
                                 'outline',
                                 text=category,
                                 category=category)
            else:
                # add a new subscription to existed category
                cat = self.body.find(f'.//*[@text="{category}"]')

            sub = SubElement(cat,
                             'outline',
                             text=site.feed.title,
                             title=site.feed.title,
                             htmlUrl=site.feed.link,
                             xmlUrl=url,
                             category=category,
                             type='rss',
                             version='RSS')

            msg = f'Subscribed "{site.feed.title}" to "{category}" category '
            print(msg)

            self._write_tree(self.root)

    def lists(self):
        # show all subscriptions by category
        return self.tree.findall('.//outline')

    def _select_message(self, items):
        return '[1{}]'.format('-' + str(len(items)) if len(items) > 1 else '')

    def _select_items(self, items, msg):
        while True:
            try:
                print()
                for i, v in enumerate(items, 1):
                    print(f'{i:^3} {v}')
                print()

                index = input(f'{msg} {self._select_message(items)}: ')
                try:
                    item = items[int(index) - 1]
                except IndexError:
                    print(f'IndexError: Input number is out of range')
                except ValueError:
                    print(f'ValueError: {index} is not a valid integer')
                else:
                    return item
                    break
            except KeyboardInterrupt:
                exit()

    def _get_urls(self, items):
        subs = []

        for sub in items:
            elem = self.tree.find(f".//outline/outline/[@xmlUrl='{sub}']")
            title = elem.get('title')
            subs.append(title)

        return subs

    def show_all(self):
        subs = self.get_subscriptions()

        if not subs:
            return False
        else:
            return subs

    def show_category(self):
        cats = self.get_categories()

        if not cats:
            return False
        else:
            urls = []
            cat = self._select_items(cats,
                                     'Choose a category you want to read')

            for node in self.tree.findall(f".//*[@text='{cat}']/outline"):
                url = node.attrib.get('xmlUrl')
                urls.append(url)

            return urls

    def show_subscription(self):
        subs = self.get_subscriptions()

        if not subs:
            return False
        else:
            urls = []
            subs_ = self._get_urls(subs)

            title = self._select_items(subs_,
                                       'Choose a title you want to read')
            sub = self.tree.find(f".//outline/outline/[@title='{title}']")
            url = sub.get('xmlUrl')
            urls.append(url)

            return urls

    def delete(self):
        cats = self.get_categories()

        if not cats:
            print('No subscriptions yet')
        else:
            cat = self._select_items(cats,
                                     'Choose a category you want to delete')
            del_cat = self.tree.find(f".//body/*[@text='{cat}']")
            self.body.remove(del_cat)

            print(f'Deleted: "{del_cat.get("text")}" category')

            self._write_tree(self.root)

    def cancel(self):
        subs = self.get_subscriptions()

        if not subs:
            print('No subscriptions yet')
        else:
            subs_ = self._get_urls(subs)

            sub = self._select_items(
                subs_, 'Choose a subscription you want to cancel')
            del_sub = self.tree.find(f".//outline/outline/[@title='{sub}']")

            cat_name = del_sub.get('category')
            cat_elem = self.tree.find(f".//body/*[@category='{cat_name}']")
            cat_elem.remove(del_sub)

            # delete category if no subscriptions exist
            if not cat_elem.find('.//'):
                self.body.remove(cat_elem)

            print(f'Canceled: {del_sub.get("text")}')

            self._write_tree(self.root)
